@@ -48,12 +48,8 @@
 
 static void help(void)
 {
-	struct ts_lib_version_data *ver = ts_libversion();
-
-	print_ascii_logo();
-
-	printf("tslib %s / libts ABI version %d (0x%06X)\n",
-		ver->package_version, ver->version_num >> 16, ver->version_num);
+	ts_print_ascii_logo(16);
+	printf("%s", tslib_version());
 	printf("\n");
 	printf("Usage: ts_test_mt [-v] [-i <device>] [-j <slots>]\n");
 	printf("\n");
