@@ -103,6 +103,12 @@
 #define UINPUT_VERSION	2
 #endif
 
+#ifndef UI_GET_SYSNAME
+#define UI_GET_SYSNAME(len)     _IOC(_IOC_READ, UINPUT_IOCTL_BASE, 44, len)
+#endif
+
+#define UINPUT_VERSION_HAVE_SYSNAME 4
+
 static char *defaultfbdevice = "/dev/fb0";
 
 struct data_t {
@@ -402,6 +408,8 @@ static int setup_uinput(struct data_t *data, int *max_slots)
 	int max_x = 0;
 	int max_y = 0;
 	int ret;
+	int have_abs_mt_position = 0;
+	int have_abs_mt_slot = 0;
 
 	if (data->nofb)
 		ret = get_abs_max_input(data, &max_x, &max_y);
@@ -476,6 +484,7 @@ static int setup_uinput(struct data_t *data, int *max_slots)
 						} else if (j == ABS_MT_POSITION_X) {
 							uidev.absmin[ABS_MT_POSITION_X] = 0;
 							uidev.absmax[ABS_MT_POSITION_X] = max_x;
+							have_abs_mt_position = 1;
 						} else if (j == ABS_MT_POSITION_Y) {
 							uidev.absmin[ABS_MT_POSITION_Y] = 0;
 							uidev.absmax[ABS_MT_POSITION_Y] = max_y;
@@ -484,14 +493,48 @@ static int setup_uinput(struct data_t *data, int *max_slots)
 							uidev.absmax[j] = absinfo.maximum;
 						}
 
-						if (j == ABS_MT_SLOT)
-							*max_slots = absinfo.maximum + 1 - absinfo.minimum;
+						if (j == ABS_MT_SLOT) {
+							if (*max_slots == 1)
+								*max_slots = absinfo.maximum + 1 -
+									     absinfo.minimum;
+
+							have_abs_mt_slot = 1;
+						}
 					}
 				}
 			}
 		}
 	}
 
+	/*
+	 *if we have multitouch we generate type B only and need this in
+	 * case of type A input
+	 */
+	if (have_abs_mt_position && !have_abs_mt_slot) {
+		if (ioctl(data->fd_uinput, UI_SET_ABSBIT, ABS_MT_TRACKING_ID) < 0) {
+			perror("ioctl UI_SET_ABSBIT");
+			goto err;
+		}
+		if (ioctl(data->fd_uinput, UI_SET_ABSBIT, ABS_MT_SLOT) < 0) {
+			perror("ioctl UI_SET_ABSBIT");
+			goto err;
+		}
+
+		/* if no user setting, we use 5 slots for type A devices */
+		if (*max_slots == 1)
+			*max_slots = 5;
+
+		if (data->verbose)
+			printf(DEFAULT_UINPUT_NAME ": We use a " GREEN
+			       "multitouch type A" RESET " device\n");
+	}
+
+	if (have_abs_mt_position && have_abs_mt_slot && data->verbose)
+		printf(DEFAULT_UINPUT_NAME ": We use a " GREEN
+		       "multitouch type B" RESET " device\n");
+
+	uidev.absmax[ABS_MT_SLOT] = *max_slots - 1;
+
 	if (write(data->fd_uinput, &uidev, sizeof(uidev)) == -1) {
 		perror("write uinput_user_dev");
 		goto err;
@@ -567,9 +610,6 @@ static void cleanup(struct data_t *data)
 	if (data->ev)
 		free(data->ev);
 
-	if (data->ts)
-		ts_close(data->ts);
-
 	if (data->fd_uinput > 0) {
 		ret = ioctl(data->fd_uinput, UI_DEV_DESTROY);
 		if (ret == -1)
@@ -618,11 +658,59 @@ static char *fetch_device_node(const char *path)
 	return devnode;
 }
 
+/* return the /dev/input/eventX path of the created device */
+static char *get_new_path(struct data_t *data)
+{
+	struct dirent **namelist;
+	const char *path = "/dev/input/";
+	int ndev;
+	int fd;
+	char buf[256];
+	int ret;
+	char *devnode = NULL;
+
+	ndev = scandir(path, &namelist, is_event_device, alphasort);
+	if (ndev <= 0)
+		return NULL;
+
+	while (ndev--) {
+		if (asprintf(&devnode, "/dev/input/%s", namelist[ndev]->d_name) == -1) {
+			devnode = NULL;
+			break;
+		}
+
+		fd = open(devnode, O_RDWR);
+		if (fd == -1)
+			return NULL;
+
+		ret = ioctl(fd, EVIOCGNAME(sizeof(buf) - 1), buf);
+		if (ret < 0) {
+			close(fd);
+			free(devnode);
+			break;
+		}
+
+		ret = strncmp(buf, data->uinput_name, strlen(data->uinput_name));
+		if (ret == 0) {
+			close(fd);
+			free(namelist[ndev]);
+			break;
+		}
+
+		close(fd);
+		free(namelist[ndev]);
+		free(devnode);
+	}
+
+	free(namelist);
+	return devnode;
+}
+
 int main(int argc, char **argv)
 {
 	struct data_t data = {
-		.fd_uinput = 0,
-		.fd_input = 0,
+		.fd_uinput = -1,
+		.fd_input = -1,
 		.uinput_name = NULL,
 		.input_name = NULL,
 		.fb_name = NULL,
@@ -805,40 +893,38 @@ int main(int argc, char **argv)
 		       ": using input device " GREEN "%s" RESET "\n",
 		       dev_input_name);
 
-	if (setup_uinput(&data, &data.slots)) {
+	if (setup_uinput(&data, &data.slots))
 		goto out;
-	} else {
-		if (data.verbose && data.slots == 1)
-			printf(DEFAULT_UINPUT_NAME
-			       ": We don't use a multitouch device\n");
-		else if (data.verbose && data.slots > 1)
-			printf(DEFAULT_UINPUT_NAME
-			       ": We use a "
-			       GREEN "multitouch" RESET
-			       " device\n");
-	}
 
 	if (data.verbose) {
 		printf(DEFAULT_UINPUT_NAME ": running uinput version %d\n", UINPUT_VERSION);
-		#if UINPUT_VERSION >= 4
-			char name[64];
-			int ret = ioctl(data.fd_uinput,
-					UI_GET_SYSNAME(sizeof(name)),
-					name);
-			if (ret >= 0) {
-				char buf[sizeof(SYS_INPUT_DIR) + sizeof(name)] = SYS_INPUT_DIR;
-				char *devnode;
-
-				snprintf(&buf[strlen(SYS_INPUT_DIR)], sizeof(name), "%s", name);
-				fprintf(stdout, "created %s\n", buf);
-				devnode = fetch_device_node(buf);
-				if (devnode)
-					fprintf(stdout, "%s\n", devnode);
+		char *devnode;
+		char name[64];
+		int ret = ioctl(data.fd_uinput,
+				UI_GET_SYSNAME(sizeof(name)),
+				name);
+		if (ret == -1) {
+			if (errno != EINVAL) {
+				perror("ioctl UI_GET_SYSNAME");
+				goto out;
 			}
-		#else
-			fprintf(stderr, DEFAULT_UINPUT_NAME
-				": See the kernel log for the device number\n");
-		#endif
+
+			/* assume we have UINPUT_VERSION < 4 */
+
+			devnode = get_new_path(&data);
+			if (!devnode)
+				goto out;
+
+			fprintf(stdout, "%s\n", devnode);
+		} else {
+			char buf[sizeof(SYS_INPUT_DIR) + sizeof(name)] = SYS_INPUT_DIR;
+
+			snprintf(&buf[strlen(SYS_INPUT_DIR)], sizeof(name), "%s", name);
+			fprintf(stdout, "created %s\n", buf);
+			devnode = fetch_device_node(buf);
+			if (devnode)
+				fprintf(stdout, "%s\n", devnode);
+		}
 	}
 
 	data.ev = malloc(sizeof(struct input_event) * MAX_CODES_PER_SLOT * data.slots);
@@ -862,15 +948,29 @@ int main(int argc, char **argv)
 	}
 
 	if (run_daemon) {
-	#if UINPUT_VERSION >= 4
+		char *devnode;
 		char name[64];
 		int ret = ioctl(data.fd_uinput,
 				UI_GET_SYSNAME(sizeof(name)),
 				name);
-		if (ret >= 0) {
+		if (ret == -1) {
+			if (errno != EINVAL) {
+				perror("ioctl UI_GET_SYSNAME");
+				goto out;
+			}
+
+			/* assume we have UINPUT_VERSION < 4 */
+
+			if (data.verbose_daemon) {
+				devnode = get_new_path(&data);
+				if (!devnode)
+					goto out;
+
+				fprintf(stdout, "%s\n", devnode);
+			}
+		} else {
 			if (data.verbose_daemon) {
 				char buf[sizeof(SYS_INPUT_DIR) + sizeof(name)] = SYS_INPUT_DIR;
-				char *devnode;
 
 				snprintf(&buf[strlen(SYS_INPUT_DIR)], sizeof(name), "%s", name);
 				devnode = fetch_device_node(buf);
@@ -879,17 +979,10 @@ int main(int argc, char **argv)
 			} else {
 				fprintf(stdout, "%s\n", name);
 			}
-
-			fflush(stdout);
-		} else {
-			perror("ioctl UI_GET_SYSNAME");
-			goto out;
 		}
-	#else
-		fprintf(stderr, DEFAULT_UINPUT_NAME
-			": kernel with uinput version < 4 is not supported\n");
-		goto out;
-	#endif
+
+		fflush(stdout);
+
 		if (daemon(0, 0) == -1) {
 			perror("error starting daemon");
 			goto out;
